<?php

require_once 'db.php';
$db = new db();

function f_get_contents(string $url, int $timeout=10, $post_data=null, bool $header=false, bool $ebarter_bot=false, array $headers_request=null, $method=null, $user_agent='Mozilla/5.0 (Windows 10) Gecko/20100101 Firefox/83.0')
{
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
    curl_setopt($ch, CURLOPT_USERAGENT, $user_agent);
    if ($post_data) {
        curl_setopt($ch, CURLOPT_POSTFIELDS, $post_data);
    }
    if ($header) {
        curl_setopt($ch, CURLOPT_HEADER, true);
    }
    if ($ebarter_bot) {
        curl_setopt($ch, CURLOPT_USERAGENT, 'ebarter_bot');
    }
    if ($headers_request) {
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers_request);
    }
    if ($method) {
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
    }
    $out = curl_exec($ch);
    curl_close($ch);

    return $out;
}

function parseHtmlOto($html)
{
    $status = preg_match('/Аттестат действителен/', $html);
    preg_match('/div class=\"title\">(.*)<.*Директор.*p>(.*)<\/p.*Полное наименование.*p>(.*)<\/p>.*ОГРН.*<p>(.*)<\/p.*Орг\. форма.*<p>(.*)<\/p>.*Телефон.*<p>(.*)<\/p>.*mailto\:(.*)\".*Сайт.*href=\"(.*)\"/Usi', $html, $mainData);
    preg_match_all('/tbody.*tr.*td.*href=\"#pto=(\d{1,6})\".*status (.*)\".*td.*a.*>(.*)<\/a>.*td.*<a.*>(.*)<\/.*td.*<a.*>(.*)<\//Usi', $html, $ptos);

    $resultMainData = [
        'name' => $mainData[1],
        'directorName' => $mainData[2],
        'fullName' => $mainData[3],
        'EGRN' => $mainData[4],
        'form' => $mainData[5],
        'phone' => $mainData[6],
        'email' => $mainData[7],
        'site' => $mainData[8],
    ];

    preg_match('/Адрес регистрации.*<p>(.*)<\//Usi', $html, $outAddress);

    if (empty($outAddress[1])) {
        // значит ЮЛ
        preg_match('/Aдрес юридический.*<p>(.*)<.*Адрес почтовый.*<p>(.*)<\//Usi', $html, $outAddress);
        $resultMainData['fisicalAddress'] = $outAddress[2];
        $resultMainData['legalAddress'] = $outAddress[1];
    } else {
        $resultMainData['fisicalAddress'] = $outAddress[1];
        $resultMainData['legalAddress'] = null;
    }

    $resultPtos = (function () use ($ptos) {
        $result = [];
        foreach ($ptos[1] as $key=>$value) {
            $result[] = [
                'id' => $ptos[1][$key],
                'status' => $ptos[2][$key] == 'ok',
                'address' => $ptos[3][$key],
                'limit' => $ptos[4][$key],
                'attestation' => $ptos[5][$key],
            ];
        }

        return $result;
    })();

    $result = [
        'status' => $status,
        'mainData' => $resultMainData,
        'ptos' => $resultPtos
    ];

    return $result;
}

function writeDataOto(string $reestr, array $data)
{
    global $db;

    $idOto = $db->query('SELECT id FROM otos WHERE reestr = ?',
        $reestr)->assoc()['id'] ?? null;

    if ($idOto) {
        $db->query('UPDATE otos SET 
                    phone = ?,
                    email = ?,
                    site = ?,
                    status = ?,
                    fisical_address = ?,
                    legal_address = ?
                WHERE reestr = ?',
            $data['mainData']['phone'],
            $data['mainData']['email'],
            $data['mainData']['site'],
            $data['status'],
            $data['mainData']['fisicalAddress'],
            $data['mainData']['legalAddress'],
            $reestr,
        );

        foreach ($data['ptos'] as $pto) {

            $attestation = array_flip(explode(', ', $pto['attestation']));

            $idPto = $db->query('SELECT id FROM ptos WHERE _id = ?',
                $pto['id'])->assoc()['id'] ?? null;

            if ($idPto) {
                $db->query('UPDATE ptos SET 
                                status = ?,
                                address = ?,
                                limit_dk = ?,
                                L = ?,
                                M1 = ?,
                                M2 = ?,
                                M3 = ?,
                                N1 = ?,
                                N2 = ?,
                                N3 = ?,
                                O1 = ?,
                                O2 = ?,
                                O3 = ?,
                                O4 = ?
                            WHERE id = ?',
                    $pto['status'],
                    $pto['address'],
                    $pto['limit'],
                    (int) ((bool) ($attestation['L'] ?? null)),
                    (int) ((bool) ($attestation['M1'] ?? null)),
                    (int) ((bool) ($attestation['M2'] ?? null)),
                    (int) ((bool) ($attestation['M3'] ?? null)),
                    (int) ((bool) ($attestation['N1'] ?? null)),
                    (int) ((bool) ($attestation['N2'] ?? null)),
                    (int) ((bool) ($attestation['N3'] ?? null)),
                    (int) ((bool) ($attestation['O1'] ?? null)),
                    (int) ((bool) ($attestation['O2'] ?? null)),
                    (int) ((bool) ($attestation['O3'] ?? null)),
                    (int) ((bool) ($attestation['O4'] ?? null)),
                    $idPto
                );
            } else {
                $db->query('INSERT INTO ptos (id_oto, _id, status, address, limit_dk, L, M1, M2, M3, N1, N2, N3, O1, O2, O3, O4) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)',
                    $idOto,
                    $pto['id'],
                    $pto['status'],
                    $pto['address'],
                    $pto['limit'],
                    (int) ((bool) ($attestation['L'] ?? null)),
                    (int) ((bool) ($attestation['M1'] ?? null)),
                    (int) ((bool) ($attestation['M2'] ?? null)),
                    (int) ((bool) ($attestation['M3'] ?? null)),
                    (int) ((bool) ($attestation['N1'] ?? null)),
                    (int) ((bool) ($attestation['N2'] ?? null)),
                    (int) ((bool) ($attestation['N3'] ?? null)),
                    (int) ((bool) ($attestation['O1'] ?? null)),
                    (int) ((bool) ($attestation['O2'] ?? null)),
                    (int) ((bool) ($attestation['O3'] ?? null)),
                    (int) ((bool) ($attestation['O4'] ?? null)),
                );
            }

        }

    } else {
        $idOto = $db->query('INSERT INTO otos (reestr, status, name, full_name, egrn, form, phone, email, site, fisical_address, legal_address) VALUES (?,?,?,?,?,?,?,?,?,?,?)',
            $reestr,
            $data['status'],
            $data['mainData']['name'],
            $data['mainData']['fullName'],
            $data['mainData']['EGRN'],
            $data['mainData']['form'],
            $data['mainData']['phone'],
            $data['mainData']['email'],
            $data['mainData']['site'],
            $data['mainData']['fisicalAddress'],
            $data['mainData']['legalAddress'],
        )->get_id();

        foreach ($data['ptos'] as $pto) {

            $attestation = array_flip(explode(', ', $pto['attestation']));

            $db->query('INSERT INTO ptos (id_oto, _id, status, address, limit_dk, L, M1, M2, M3, N1, N2, N3, O1, O2, O3, O4) 
                        VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)',
                $idOto,
                $pto['id'],
                $pto['status'],
                $pto['address'],
                $pto['limit'],
                (int) ((bool) ($attestation['L'] ?? null)),
                (int) ((bool) ($attestation['M1'] ?? null)),
                (int) ((bool) ($attestation['M2'] ?? null)),
                (int) ((bool) ($attestation['M3'] ?? null)),
                (int) ((bool) ($attestation['N1'] ?? null)),
                (int) ((bool) ($attestation['N2'] ?? null)),
                (int) ((bool) ($attestation['N3'] ?? null)),
                (int) ((bool) ($attestation['O1'] ?? null)),
                (int) ((bool) ($attestation['O2'] ?? null)),
                (int) ((bool) ($attestation['O3'] ?? null)),
                (int) ((bool) ($attestation['O4'] ?? null)),
            );
        }
    }
}


function parseFirstPageHtml($html)
{
    preg_match('/Set-Cookie: (.*);/Usi', $html, $outCookie);
    $cookie = $outCookie[1] ?? null;

    preg_match('/id=\"j_id1:javax.faces.ViewState:0\" value=\"(.*)\"/Usi', $html, $outViewState);
    $viewState = $outViewState[1];

    preg_match_all('/data-expanded=\"true\".*footable-first-visible.*href=\"#oto=([\d]{1,6})\".*status (.*)\".*item-name.*>(.*)<\/.*a href.*>(.*)<.*a href.*>(.*)<.*a href.*>(.*)<.*a href.*>(.*)<.*a href.*>(.*)<\/a/Usi', $html, $outOtos);
    $otos = (function () use ($outOtos) {
        $result = [];
        foreach ($outOtos[1] as $key=>$value) {
            $result[] = [
                'reestr' => $outOtos[1][$key],
                'status' => $outOtos[2][$key] == 'ok',
                'name' => $outOtos[3][$key],
                'address' => $outOtos[4][$key],
                'phone' => $outOtos[5][$key],
                'email' => $outOtos[6][$key],
                'site' => $outOtos[7][$key],
            ];
        }

        return $result;
    })();

    preg_match('/j_idt44.*>([\d]{1,4})</', $html, $outCountPages);
    $countPages = $outCountPages[1];

    preg_match('/mainForm:j_idt([\d]{1,4}):/Usi', $html, $outTypePage);
    $typePage = $outTypePage[1];

    preg_match('/search-input.*mainForm:j_idt([\d]{1,3}).*mainForm:j_idt([\d]{1,3}).*mainForm:j_idt([\d]{1,3}).*mainForm:j_idt([\d]{1,3})/', $html, $outIds);
    $ids = [
        $outIds[1],
        $outIds[2],
        $outIds[3],
        $outIds[4],
    ];

    $result = [
        'viewState' => $viewState,
        'otos' => $otos,
        'cookie' => $cookie,
        'countPages' => $countPages,
        'typePage' => $typePage,
        'ids' => $ids
    ];

    return $result;
}

function parseOtherPageHtml($html)
{
    preg_match('/Set-Cookie: (.*);/Usi', $html, $outCookie);
    $cookie = $outCookie[1] ?? null;

    preg_match('/id=\"j_id1:javax.faces.ViewState:0\".*DATA.*\[(.*)\]/Usi', $html, $outViewState);
    $viewState = $outViewState[1];

    preg_match_all('/data-expanded=\"true\".*footable-first-visible.*href=\"#oto=([\d]{1,6})\".*status (.*)\".*item-name.*>(.*)<\/.*a href.*>(.*)<.*a href.*>(.*)<.*a href.*>(.*)<.*a href.*>(.*)<.*a href.*>(.*)<\/a/Usi', $html, $outOtos);
    $otos = (function () use ($outOtos) {
        $result = [];
        foreach ($outOtos[1] as $key=>$value) {
            $result[] = [
                'reestr' => $outOtos[1][$key],
                'status' => $outOtos[2][$key] == 'ok',
                'name' => $outOtos[3][$key],
                'address' => $outOtos[4][$key],
                'phone' => $outOtos[5][$key],
                'email' => $outOtos[6][$key],
                'site' => $outOtos[7][$key],
            ];
        }

        return $result;
    })();

    preg_match('/j_idt44.*>([\d]{1,4})</', $html, $outCountPages);
    $countPages = $outCountPages[1];

    preg_match('/mainForm:j_idt([\d]{1,4}):/Usi', $html, $outTypePage);
    $typePage = $outTypePage[1];

    $result = [
        'viewState' => $viewState,
        'otos' => $otos,
        'cookie' => $cookie,
        'countPages' => $countPages,
        'typePage' => $typePage,
    ];

    return $result;
}

function writePageData($otos, $ids=null): void
{
    global $db;

    foreach ($otos as $oto) {
        $id = $db->query('SELECT id FROM otos_short_data WHERE reestr = ?',
            $oto['reestr'])->assoc()['id'] ?? null;

        if ($id) {
            $db->query('UPDATE otos_short_data SET status = ?, phone = ?, email = ?, site = ? WHERE id = ?',
                (int) ($oto['status'] == 'ok'),
                $oto['phone'],
                $oto['email'],
                $oto['site'],
                $id,
            );
        } else {
            $db->query('INSERT INTO otos_short_data (status, reestr, name, phone, email, site) VALUES (?,?,?,?,?,?)',
                (int) ($oto['status'] == 'ok'),
                $oto['reestr'],
                $oto['name'],
                $oto['phone'],
                $oto['email'],
                $oto['site'],
            );
        }
    }

}

function getConfig()
{
    global $db;
    $result = [];
    $sql = $db->query('SELECT `key`, `value` FROM config');

    while ($one = $sql->assoc()) {
        $result[$one['key']] = $one['value'];
    }
    return $result;
}

function setConfig($array, ?array $ids=null)
{
    global $db;

    if (!empty($array['cookie'])) {
        $db->query('UPDATE config SET `value` = ? WHERE `key` = "cookie"',
            $array['cookie']);
    }

    if (!empty($array['viewState'])) {
        $db->query('UPDATE config SET `value` = ? WHERE `key` = "viewState"',
            $array['viewState']);
    }

    if ($ids) {
        $check = $db->query('SELECT id FROM config WHERE `key` = "ids_for_one_oto"')->assoc()['id'] ?? null;
        if ($check) {
            $db->query('UPDATE config SET `value` = ? WHERE `key` = "ids_for_one_oto"',
                json_encode($ids));
        } else {
            $db->query('INSERT INTO config (`key`,`value`) VALUES (?,?)',
                'ids_for_one_oto', json_encode($ids));
        }
    }
}

function updateConfig()
{
    $firstPage = (function () {
        return f_get_contents('https://oto-register.autoins.ru/oto/', header: true);
    })();
    $pageData = parseFirstPageHtml($firstPage);
    writePageData($pageData['otos']);
    setConfig($pageData, $pageData['ids']);
}

function getOTO($reestr, $viewState, $cookie, $ids)
{
    $postData = 'javax.faces.partial.ajax=true&javax.faces.source=mainForm%3AremoteCommandSetupOtoId_action&javax.faces.partial.execute=mainForm%3AremoteCommandSetupOtoId_action&javax.faces.partial.render=mainForm%3AcontentTabsPanel+mainForm%3AotoPanel&mainForm%3AremoteCommandSetupOtoId_action=mainForm%3AremoteCommandSetupOtoId_action&otoId=' . $reestr . '&mainForm=mainForm&mainForm%3Aj_idt' . $ids[0] . '=&mainForm%3Aj_idt' . $ids[1] . '=&mainForm%3Aj_idt' . $ids[2] . '=&mainForm%3Aj_idt' . $ids[3] . '=&javax.faces.ViewState=' . $viewState;
    $headersRequest = [
        'Cookie: ' . $cookie
    ];
    $response = f_get_contents('https://oto-register.autoins.ru/oto/index.xhtml', post_data: $postData, headers_request: $headersRequest, method: 'POST');
    return $response;
}

function getPto($idPto, $viewState, $cookie, $ids)
{
    $postData = 'javax.faces.partial.ajax=true&javax.faces.source=mainForm%3AremoteCommandSetupPtoId_action&javax.faces.partial.execute=mainForm%3AremoteCommandSetupPtoId_action&javax.faces.partial.render=mainForm%3AcontentTabsPanel+mainForm%3AptoPanel&mainForm%3AremoteCommandSetupPtoId_action=mainForm%3AremoteCommandSetupPtoId_action&ptoId=' . $idPto . '&mainForm=mainForm&mainForm%3Aj_idt' . $ids[0] . '=&mainForm%3Aj_idt' . $ids[1] . '=&mainForm%3Aj_idt' . $ids[2] . '=&mainForm%3Aj_idt' . $ids[3] . '=&javax.faces.ViewState=' . $viewState;
    $headersRequest = [
        'Cookie: ' . $cookie
    ];
    $response = f_get_contents('https://oto-register.autoins.ru/oto/index.xhtml', post_data: $postData, headers_request: $headersRequest, method: 'POST');
    return $response;
}

function parseHtmlPto($html): array
{
    preg_match('/Долгота.*tbody.*td>(.*)<\/.*<td>(.*)<\//Usi', $html, $coordinatesOut);

    $result = [
        'lat' => $coordinatesOut[1],
        'lon' => $coordinatesOut[2],
    ];

    return $result;
}

function updatePtoCoordinates($_id, $ptoData)
{
    global $db;
    $db->query('UPDATE ptos SET lat = ?, lon = ? WHERE _id = ?',
        $ptoData['lat'],
        $ptoData['lon'],
        $_id,
    );
}